/* xdwapi sample 4

Function :
  Converts an application file to a DocuWorks document.
How to use :
  xdwsmpl4

  The dialog box is opened.
  Set application file at input file and DocuWorks document at output file.
  If Convert button is clicked, the converting is started.
*/

#include <windows.h>
#include <xdw_api.h>
#include "resource.h"

HINSTANCE ghInstance = NULL;
XDW_CREATE_HANDLE hXdwCreate = NULL;

void ErrorMessage(HWND hWnd, int code)
{
	switch (code) {
	case XDW_E_NOT_INSTALLED:
		SetDlgItemText(
			hWnd, IDC_STATUS, "DocuWorks not installed.");
		break;
	case XDW_E_FILE_NOT_FOUND:
		SetDlgItemText(
			hWnd, IDC_STATUS, "File not found.");
		break;
	case XDW_E_FILE_EXISTS:
		SetDlgItemText(
			hWnd, IDC_STATUS, "File exists.");
		break;
	case XDW_E_ACCESSDENIED:
	case XDW_E_INVALID_ACCESS:
	case XDW_E_INVALID_NAME:
	case XDW_E_BAD_NETPATH:
		SetDlgItemText(
			hWnd, IDC_STATUS, "File not opened.");
		break;
	default:
		SetDlgItemText(
			hWnd, IDC_STATUS, "Error.");
		break;
	}
}

VOID CALLBACK TimerProc(HWND hWnd, UINT uMsg, UINT idEvent, DWORD dwTime)
{
	XDW_CREATE_STATUS status;
	status.nSize = sizeof(XDW_CREATE_STATUS);

	// Get the status of converting the application file. 
	XDW_GetStatusCreationFromAppFile(hXdwCreate, &status);

	switch (status.phase) {
	case XDW_CRTP_BEGINNING:
		SetDlgItemText(hWnd, IDC_STATUS, "Preparing for creation process.");
		break;
	case XDW_CRTP_PRINTING:
		SetDlgItemText(hWnd, IDC_STATUS, "Printing on the application");
		break;
	case XDW_CRTP_PAGE_CREATING:
		char message[100];
		wsprintf(
			message, "Creating a page. (%d/%d)",
			status.nPage, status.nTotalPage);
		SetDlgItemText(hWnd, IDC_STATUS, message);
		break;
	case XDW_CRTP_ORIGINAL_APPENDING:
		SetDlgItemText(hWnd, IDC_STATUS, "Attaching the original data.");
		break;
	case XDW_CRTP_WRITING:
		SetDlgItemText(hWnd, IDC_STATUS, "Writing to the file.");
		break;
	case XDW_CRTP_CANCELING:
		SetDlgItemText(hWnd, IDC_STATUS, "During cancellation process.");
		break;
	case XDW_CRTP_FINISHED:
		{
			// Terminate converting an application file.
			int result = XDW_EndCreationFromAppFile(hXdwCreate, NULL);
			if (result < 0){
				SetDlgItemText(hWnd, IDC_STATUS, "Converting failed.");
			}
			else {
				SetDlgItemText(hWnd, IDC_STATUS, "Processing completed.");
			}
			hXdwCreate = NULL;
			break;
		}
	case XDW_CRTP_CANCELED:
		SetDlgItemText(hWnd, IDC_STATUS, "Cancellation completed.");
		// Terminate converting an application file.
		XDW_EndCreationFromAppFile(hXdwCreate, NULL);
		hXdwCreate = NULL;
		break;
	}

	if (hXdwCreate == NULL) {
		SetDlgItemText(hWnd, IDCANCEL, "Close");
		EnableWindow(GetDlgItem(hWnd, IDOK), TRUE);
		KillTimer(hWnd, 1);
	}
}

void DoTransfer(HWND hWnd)
{
	char src[MAX_PATH], dst[MAX_PATH];
	GetDlgItemText(hWnd, IDC_SRCPATH, src, MAX_PATH);
	GetDlgItemText(hWnd, IDC_DSTPATH, dst, MAX_PATH);

	char srcPath[MAX_PATH], dstPath[MAX_PATH];
	GetFullPathName(src, MAX_PATH, srcPath, NULL);
	GetFullPathName(dst, MAX_PATH, dstPath, NULL);

	BOOL bWithOriginal = IsDlgButtonChecked(hWnd, IDC_WITHORG);

	// Start converting the application file to the DocuWorks document.
	int result = XDW_BeginCreationFromAppFile(
		srcPath, dstPath, bWithOriginal, &hXdwCreate, NULL);

	if (result < 0) {
		ErrorMessage(hWnd, result);
		return;
	}

	SetDlgItemText(hWnd, IDCANCEL, "Cancel");
	EnableWindow(GetDlgItem(hWnd, IDOK), FALSE);
	SetTimer(hWnd, 1, 200, (TIMERPROC)TimerProc);
}

void CancelTransfer(HWND hWnd)
{
	// Cancel converting the application file to the DocuWorks document.
	int result = XDW_CancelCreationFromAppFile(hXdwCreate, NULL);

	if (result < 0) {
		ErrorMessage(hWnd, result);
		return;
	}
}

void SrcFileDialog(HWND hWnd)
{
	SetDlgItemText(hWnd, IDC_STATUS, "");
	char path[MAX_PATH] = "";
	OPENFILENAME ofn = {
		sizeof(OPENFILENAME), hWnd, ghInstance, "All files (*.*)\0*.*\0",
		NULL, 0, 1, path, MAX_PATH, NULL, 0, NULL, "input file",
		OFN_FILEMUSTEXIST, 0, 0, 0, 0, NULL, NULL };
	if (GetOpenFileName(&ofn)) {
		SetDlgItemText(hWnd, IDC_SRCPATH, path);
	}
}

void DstFileDialog(HWND hWnd)
{
	SetDlgItemText(hWnd, IDC_STATUS, "");
	char path[MAX_PATH] = "";
	OPENFILENAME ofn = {
		sizeof(OPENFILENAME), hWnd, ghInstance,
		"DocuWorks document (*.xdw)\0*.xdw\0", NULL, 0, 1, path, MAX_PATH, NULL, 0,
		NULL, "output file", OFN_PATHMUSTEXIST, 0, 0, 0, 0, NULL, NULL };
	if (GetSaveFileName(&ofn)) {
		SetDlgItemText(hWnd, IDC_DSTPATH, path);
	}
}

BOOL CALLBACK DialogProc(
	HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if (uMsg == WM_COMMAND) {
		switch (wParam) {
		case IDC_SRCBTN:
			SrcFileDialog(hWnd);
			return TRUE;
		case IDC_DSTBTN:
			DstFileDialog(hWnd);
			return TRUE;
		case IDOK:
			if (hXdwCreate == NULL) {
				DoTransfer(hWnd);
			}
			return TRUE;
		case IDCANCEL:
			if (hXdwCreate == NULL) {
				EndDialog(hWnd, 0);
			} else {
				CancelTransfer(hWnd);
			}
			return TRUE;
		}
	}
	return FALSE;
}

int WINAPI WinMain(
	HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpszCmdLine,
	int nCmdShow)
{
	ghInstance = hInstance;
	DialogBox(
		hInstance, MAKEINTRESOURCE(IDD_DIALOG1), NULL,
		(DLGPROC)DialogProc);
	return NULL;
}
